@@ -2,22 +2,28 @@
 
 class ResourceLoaderStartUpModuleTest extends ResourceLoaderTestCase {
 
+	protected static function expandPlaceholders( $text ) {
+		return strtr( $text, [
+			'{blankVer}' => self::BLANK_VERSION
+		] );
+	}
+
 	public static function provideGetModuleRegistrations() {
-		return array(
-			array( array(
+		return [
+			[ [
 				'msg' => 'Empty registry',
-				'modules' => array(),
+				'modules' => [],
 				'out' => '
 mw.loader.addSource( {
     "local": "/w/load.php"
 } );
 mw.loader.register( [] );'
-			) ),
-			array( array(
+			] ],
+			[ [
 				'msg' => 'Basic registry',
-				'modules' => array(
+				'modules' => [
 					'test.blank' => new ResourceLoaderTestModule(),
-				),
+				],
 				'out' => '
 mw.loader.addSource( {
     "local": "/w/load.php"
@@ -25,17 +31,17 @@ mw.loader.addSource( {
 mw.loader.register( [
     [
         "test.blank",
-        "wvTifjse"
+        "{blankVer}"
     ]
 ] );',
-			) ),
-			array( array(
+			] ],
+			[ [
 				'msg' => 'Group signature',
-				'modules' => array(
+				'modules' => [
 					'test.blank' => new ResourceLoaderTestModule(),
-					'test.group.foo' => new ResourceLoaderTestModule( array( 'group' => 'x-foo' ) ),
-					'test.group.bar' => new ResourceLoaderTestModule( array( 'group' => 'x-bar' ) ),
-				),
+					'test.group.foo' => new ResourceLoaderTestModule( [ 'group' => 'x-foo' ] ),
+					'test.group.bar' => new ResourceLoaderTestModule( [ 'group' => 'x-bar' ] ),
+				],
 				'out' => '
 mw.loader.addSource( {
     "local": "/w/load.php"
@@ -43,28 +49,28 @@ mw.loader.addSource( {
 mw.loader.register( [
     [
         "test.blank",
-        "wvTifjse"
+        "{blankVer}"
     ],
     [
         "test.group.foo",
-        "wvTifjse",
+        "{blankVer}",
         [],
         "x-foo"
     ],
     [
         "test.group.bar",
-        "wvTifjse",
+        "{blankVer}",
         [],
         "x-bar"
     ]
 ] );'
-			) ),
-			array( array(
+			] ],
+			[ [
 				'msg' => 'Different target (non-test should not be registered)',
-				'modules' => array(
+				'modules' => [
 					'test.blank' => new ResourceLoaderTestModule(),
-					'test.target.foo' => new ResourceLoaderTestModule( array( 'targets' => array( 'x-foo' ) ) ),
-				),
+					'test.target.foo' => new ResourceLoaderTestModule( [ 'targets' => [ 'x-foo' ] ] ),
+				],
 				'out' => '
 mw.loader.addSource( {
     "local": "/w/load.php"
@@ -72,21 +78,21 @@ mw.loader.addSource( {
 mw.loader.register( [
     [
         "test.blank",
-        "wvTifjse"
+        "{blankVer}"
     ]
 ] );'
-			) ),
-			array( array(
+			] ],
+			[ [
 				'msg' => 'Foreign source',
-				'sources' => array(
-					'example' => array(
+				'sources' => [
+					'example' => [
 						'loadScript' => 'http://example.org/w/load.php',
 						'apiScript' => 'http://example.org/w/api.php',
-					),
-				),
-				'modules' => array(
-					'test.blank' => new ResourceLoaderTestModule( array( 'source' => 'example' ) ),
-				),
+					],
+				],
+				'modules' => [
+					'test.blank' => new ResourceLoaderTestModule( [ 'source' => 'example' ] ),
+				],
 				'out' => '
 mw.loader.addSource( {
     "local": "/w/load.php",
@@ -95,36 +101,36 @@ mw.loader.addSource( {
 mw.loader.register( [
     [
         "test.blank",
-        "wvTifjse",
+        "{blankVer}",
         [],
         null,
         "example"
     ]
 ] );'
-			) ),
-			array( array(
+			] ],
+			[ [
 				'msg' => 'Conditional dependency function',
-				'modules' => array(
+				'modules' => [
 					'test.x.core' => new ResourceLoaderTestModule(),
-					'test.x.polyfill' => new ResourceLoaderTestModule( array(
+					'test.x.polyfill' => new ResourceLoaderTestModule( [
 						'skipFunction' => 'return true;'
-					) ),
-					'test.y.polyfill' => new ResourceLoaderTestModule( array(
+					] ),
+					'test.y.polyfill' => new ResourceLoaderTestModule( [
 						'skipFunction' =>
 							'return !!(' .
 							'    window.JSON &&' .
 							'    JSON.parse &&' .
 							'    JSON.stringify' .
 							');'
-					) ),
-					'test.z.foo' => new ResourceLoaderTestModule( array(
-						'dependencies' => array(
+					] ),
+					'test.z.foo' => new ResourceLoaderTestModule( [
+						'dependencies' => [
 							'test.x.core',
 							'test.x.polyfill',
 							'test.y.polyfill',
-						),
-					) ),
-				),
+						],
+					] ),
+				],
 				'out' => '
 mw.loader.addSource( {
     "local": "/w/load.php"
@@ -132,11 +138,11 @@ mw.loader.addSource( {
 mw.loader.register( [
     [
         "test.x.core",
-        "wvTifjse"
+        "{blankVer}"
     ],
     [
         "test.x.polyfill",
-        "wvTifjse",
+        "{blankVer}",
         [],
         null,
         null,
@@ -144,7 +150,7 @@ mw.loader.register( [
     ],
     [
         "test.y.polyfill",
-        "wvTifjse",
+        "{blankVer}",
         [],
         null,
         null,
@@ -152,7 +158,7 @@ mw.loader.register( [
     ],
     [
         "test.z.foo",
-        "wvTifjse",
+        "{blankVer}",
         [
             0,
             1,
@@ -160,67 +166,67 @@ mw.loader.register( [
         ]
     ]
 ] );',
-			) ),
-			array( array(
+			] ],
+			[ [
 				// This may seem like an edge case, but a plain MediaWiki core install
 				// with a few extensions installed is likely far more complex than this
 				// even, not to mention an install like Wikipedia.
 				// TODO: Make this even more realistic.
 				'msg' => 'Advanced (everything combined)',
-				'sources' => array(
-					'example' => array(
+				'sources' => [
+					'example' => [
 						'loadScript' => 'http://example.org/w/load.php',
 						'apiScript' => 'http://example.org/w/api.php',
-					),
-				),
-				'modules' => array(
+					],
+				],
+				'modules' => [
 					'test.blank' => new ResourceLoaderTestModule(),
 					'test.x.core' => new ResourceLoaderTestModule(),
-					'test.x.util' => new ResourceLoaderTestModule( array(
-						'dependencies' => array(
+					'test.x.util' => new ResourceLoaderTestModule( [
+						'dependencies' => [
 							'test.x.core',
-						),
-					) ),
-					'test.x.foo' => new ResourceLoaderTestModule( array(
-						'dependencies' => array(
+						],
+					] ),
+					'test.x.foo' => new ResourceLoaderTestModule( [
+						'dependencies' => [
 							'test.x.core',
-						),
-					) ),
-					'test.x.bar' => new ResourceLoaderTestModule( array(
-						'dependencies' => array(
+						],
+					] ),
+					'test.x.bar' => new ResourceLoaderTestModule( [
+						'dependencies' => [
 							'test.x.core',
 							'test.x.util',
-						),
-					) ),
-					'test.x.quux' => new ResourceLoaderTestModule( array(
-						'dependencies' => array(
+						],
+					] ),
+					'test.x.quux' => new ResourceLoaderTestModule( [
+						'dependencies' => [
 							'test.x.foo',
 							'test.x.bar',
 							'test.x.util',
 							'test.x.unknown',
-						),
-					) ),
-					'test.group.foo.1' => new ResourceLoaderTestModule( array(
+						],
+					] ),
+					'test.group.foo.1' => new ResourceLoaderTestModule( [
 						'group' => 'x-foo',
-					) ),
-					'test.group.foo.2' => new ResourceLoaderTestModule( array(
+					] ),
+					'test.group.foo.2' => new ResourceLoaderTestModule( [
 						'group' => 'x-foo',
-					) ),
-					'test.group.bar.1' => new ResourceLoaderTestModule( array(
+					] ),
+					'test.group.bar.1' => new ResourceLoaderTestModule( [
 						'group' => 'x-bar',
-					) ),
-					'test.group.bar.2' => new ResourceLoaderTestModule( array(
+					] ),
+					'test.group.bar.2' => new ResourceLoaderTestModule( [
 						'group' => 'x-bar',
 						'source' => 'example',
-					) ),
-					'test.target.foo' => new ResourceLoaderTestModule( array(
-						'targets' => array( 'x-foo' ),
-					) ),
-					'test.target.bar' => new ResourceLoaderTestModule( array(
+					] ),
+					'test.target.foo' => new ResourceLoaderTestModule( [
+						'targets' => [ 'x-foo' ],
+					] ),
+					'test.target.bar' => new ResourceLoaderTestModule( [
 						'source' => 'example',
-						'targets' => array( 'x-foo' ),
-					) ),
-				),
+						'targets' => [ 'x-foo' ],
+					] ),
+				],
 				'out' => '
 mw.loader.addSource( {
     "local": "/w/load.php",
@@ -229,36 +235,36 @@ mw.loader.addSource( {
 mw.loader.register( [
     [
         "test.blank",
-        "wvTifjse"
+        "{blankVer}"
     ],
     [
         "test.x.core",
-        "wvTifjse"
+        "{blankVer}"
     ],
     [
         "test.x.util",
-        "wvTifjse",
+        "{blankVer}",
         [
             1
         ]
     ],
     [
         "test.x.foo",
-        "wvTifjse",
+        "{blankVer}",
         [
             1
         ]
     ],
     [
         "test.x.bar",
-        "wvTifjse",
+        "{blankVer}",
         [
             2
         ]
     ],
     [
         "test.x.quux",
-        "wvTifjse",
+        "{blankVer}",
         [
             3,
             4,
@@ -267,39 +273,38 @@ mw.loader.register( [
     ],
     [
         "test.group.foo.1",
-        "wvTifjse",
+        "{blankVer}",
         [],
         "x-foo"
     ],
     [
         "test.group.foo.2",
-        "wvTifjse",
+        "{blankVer}",
         [],
         "x-foo"
     ],
     [
         "test.group.bar.1",
-        "wvTifjse",
+        "{blankVer}",
         [],
         "x-bar"
     ],
     [
         "test.group.bar.2",
-        "wvTifjse",
+        "{blankVer}",
         [],
         "x-bar",
         "example"
     ]
 ] );'
-			) ),
-		);
+			] ],
+		];
 	}
 
 	/**
 	 * @dataProvider provideGetModuleRegistrations
 	 * @covers ResourceLoaderStartUpModule::compileUnresolvedDependencies
 	 * @covers ResourceLoaderStartUpModule::getModuleRegistrations
-	 * @covers ResourceLoader::makeLoaderSourcesScript
 	 * @covers ResourceLoader::makeLoaderRegisterScript
 	 */
 	public function testGetModuleRegistrations( $case ) {
@@ -309,34 +314,34 @@ mw.loader.register( [
 
 		$context = $this->getResourceLoaderContext();
 		$rl = $context->getResourceLoader();
-
 		$rl->register( $case['modules'] );
-
 		$module = new ResourceLoaderStartUpModule();
+		$out = ltrim( $case['out'], "\n" );
+
 		$this->assertEquals(
-			ltrim( $case['out'], "\n" ),
+			self::expandPlaceholders( $out ),
 			$module->getModuleRegistrations( $context ),
 			$case['msg']
 		);
 	}
 
 	public static function provideRegistrations() {
-		return array(
-			array( array(
+		return [
+			[ [
 				'test.blank' => new ResourceLoaderTestModule(),
-				'test.min' => new ResourceLoaderTestModule( array(
+				'test.min' => new ResourceLoaderTestModule( [
 					'skipFunction' =>
 						'return !!(' .
 						'    window.JSON &&' .
 						'    JSON.parse &&' .
 						'    JSON.stringify' .
 						');',
-					'dependencies' => array(
+					'dependencies' => [
 						'test.blank',
-					),
-				) ),
-			) )
-		);
+					],
+				] ),
+			] ]
+		];
 	}
 	/**
 	 * @dataProvider provideRegistrations
@@ -348,13 +353,15 @@ mw.loader.register( [
 		$rl = $context->getResourceLoader();
 		$rl->register( $modules );
 		$module = new ResourceLoaderStartUpModule();
+		$out = 'mw.loader.addSource({"local":"/w/load.php"});' . "\n"
+		. 'mw.loader.register(['
+		. '["test.blank","{blankVer}"],'
+		. '["test.min","{blankVer}",[0],null,null,'
+		. '"return!!(window.JSON\u0026\u0026JSON.parse\u0026\u0026JSON.stringify);"'
+		. ']]);';
+
 		$this->assertEquals(
-'mw.loader.addSource({"local":"/w/load.php"});' . "\n"
-. 'mw.loader.register(['
-. '["test.blank","wvTifjse"],'
-. '["test.min","wvTifjse",[0],null,null,'
-. '"return!!(window.JSON\u0026\u0026JSON.parse\u0026\u0026JSON.stringify);"'
-. ']]);',
+			self::expandPlaceholders( $out ),
 			$module->getModuleRegistrations( $context ),
 			'Minified output'
 		);
@@ -368,18 +375,18 @@ mw.loader.register( [
 		$rl = $context->getResourceLoader();
 		$rl->register( $modules );
 		$module = new ResourceLoaderStartUpModule();
-		$this->assertEquals(
+		$out =
 'mw.loader.addSource( {
     "local": "/w/load.php"
 } );
 mw.loader.register( [
     [
         "test.blank",
-        "wvTifjse"
+        "{blankVer}"
     ],
     [
         "test.min",
-        "wvTifjse",
+        "{blankVer}",
         [
             0
         ],
@@ -387,7 +394,10 @@ mw.loader.register( [
         null,
         "return !!(    window.JSON \u0026\u0026    JSON.parse \u0026\u0026    JSON.stringify);"
     ]
-] );',
+] );';
+
+		$this->assertEquals(
+			self::expandPlaceholders( $out ),
 			$module->getModuleRegistrations( $context ),
 			'Unminified output'
 		);
