@@ -40,7 +40,7 @@ class ResourcesTest extends MediaWikiTestCase {
 		$data = self::getAllModules();
 		foreach ( $data['modules'] as $moduleName => $module ) {
 			$version = $module->getVersionHash( $data['context'] );
-			$this->assertEquals( 8, strlen( $version ), "$moduleName must use ResourceLoader::makeHash" );
+			$this->assertEquals( 7, strlen( $version ), "$moduleName must use ResourceLoader::makeHash" );
 		}
 	}
 
@@ -53,7 +53,7 @@ class ResourcesTest extends MediaWikiTestCase {
 	 */
 	public function testIllegalDependencies() {
 		$data = self::getAllModules();
-		$illegalDeps = array( 'jquery', 'mediawiki' );
+		$illegalDeps = [ 'jquery', 'mediawiki' ];
 
 		/** @var ResourceLoaderModule $module */
 		foreach ( $data['modules'] as $moduleName => $module ) {
@@ -86,6 +86,25 @@ class ResourcesTest extends MediaWikiTestCase {
 		}
 	}
 
+	/**
+	 * Verify that all specified messages actually exist.
+	 */
+	public function testMissingMessages() {
+		$data = self::getAllModules();
+		$validDeps = array_keys( $data['modules'] );
+		$lang = Language::factory( 'en' );
+
+		/** @var ResourceLoaderModule $module */
+		foreach ( $data['modules'] as $moduleName => $module ) {
+			foreach ( $module->getMessages() as $msgKey ) {
+				$this->assertTrue(
+					wfMessage( $msgKey )->useDatabase( false )->inLanguage( $lang )->exists(),
+					"Message '$msgKey' required by '$moduleName' must exist"
+				);
+			}
+		}
+	}
+
 	/**
 	 * Verify that all dependencies of all modules are always satisfiable with the 'targets' defined
 	 * for the involved modules.
@@ -119,14 +138,14 @@ class ResourcesTest extends MediaWikiTestCase {
 	}
 
 	/**
-	 * CSSMin::getAllLocalFileReferences should ignore url(...) expressions
+	 * CSSMin::getLocalFileReferences should ignore url(...) expressions
 	 * that have been commented out.
 	 */
 	public function testCommentedLocalFileReferences() {
 		$basepath = __DIR__ . '/../data/css/';
 		$css = file_get_contents( $basepath . 'comments.css' );
-		$files = CSSMin::getAllLocalFileReferences( $css, $basepath );
-		$expected = array( $basepath . 'not-commented.gif' );
+		$files = CSSMin::getLocalFileReferences( $css, $basepath );
+		$expected = [ $basepath . 'not-commented.gif' ];
 		$this->assertArrayEquals(
 			$expected,
 			$files,
@@ -149,7 +168,7 @@ class ResourcesTest extends MediaWikiTestCase {
 		// Initialize ResourceLoader
 		$rl = new ResourceLoader();
 
-		$modules = array();
+		$modules = [];
 
 		foreach ( $rl->getModuleNames() as $moduleName ) {
 			$modules[$moduleName] = $rl->getModule( $moduleName );
@@ -158,11 +177,11 @@ class ResourcesTest extends MediaWikiTestCase {
 		// Restore settings
 		$wgEnableJavaScriptTest = $org_wgEnableJavaScriptTest;
 
-		return array(
+		return [
 			'modules' => $modules,
 			'resourceloader' => $rl,
 			'context' => new ResourceLoaderContext( $rl, new FauxRequest() )
-		);
+		];
 	}
 
 	/**
@@ -171,7 +190,7 @@ class ResourcesTest extends MediaWikiTestCase {
 	 */
 	public static function provideMediaStylesheets() {
 		$data = self::getAllModules();
-		$cases = array();
+		$cases = [];
 
 		foreach ( $data['modules'] as $moduleName => $module ) {
 			if ( !$module instanceof ResourceLoaderFileModule ) {
@@ -193,13 +212,20 @@ class ResourcesTest extends MediaWikiTestCase {
 			foreach ( $styleFiles as $media => $files ) {
 				if ( $media && $media !== 'all' ) {
 					foreach ( $files as $file ) {
-						$cases[] = array(
+						$cases[] = [
 							$moduleName,
 							$media,
 							$file,
 							// XXX: Wrapped in an object to keep it out of PHPUnit output
-							(object)array( 'cssText' => $readStyleFile->invoke( $module, $file, $flip ) ),
-						);
+							(object)[
+								'cssText' => $readStyleFile->invoke(
+									$module,
+									$file,
+									$flip,
+									$data['context']
+								)
+							],
+						];
 					}
 				}
 			}
@@ -217,25 +243,24 @@ class ResourcesTest extends MediaWikiTestCase {
 	 */
 	public static function provideResourceFiles() {
 		$data = self::getAllModules();
-		$cases = array();
+		$cases = [];
 
 		// See also ResourceLoaderFileModule::__construct
-		$filePathProps = array(
+		$filePathProps = [
 			// Lists of file paths
-			'lists' => array(
+			'lists' => [
 				'scripts',
 				'debugScripts',
-				'loaderScripts',
 				'styles',
-			),
+			],
 
 			// Collated lists of file paths
-			'nested-lists' => array(
+			'nested-lists' => [
 				'languageScripts',
 				'skinScripts',
 				'skinStyles',
-			),
-		);
+			],
+		];
 
 		foreach ( $data['modules'] as $moduleName => $module ) {
 			if ( !$module instanceof ResourceLoaderFileModule ) {
@@ -244,7 +269,7 @@ class ResourcesTest extends MediaWikiTestCase {
 
 			$reflectedModule = new ReflectionObject( $module );
 
-			$files = array();
+			$files = [];
 
 			foreach ( $filePathProps['lists'] as $propName ) {
 				$property = $reflectedModule->getProperty( $propName );
@@ -286,11 +311,11 @@ class ResourcesTest extends MediaWikiTestCase {
 
 			// Populate cases
 			foreach ( $files as $file ) {
-				$cases[] = array(
+				$cases[] = [
 					$method->invoke( $module, $file ),
 					$moduleName,
 					( $file instanceof ResourceLoaderFilePath ? $file->getPath() : $file ),
-				);
+				];
 			}
 
 			// To populate missingLocalFileRefs. Not sure how sane this is inside this test...
@@ -305,11 +330,11 @@ class ResourcesTest extends MediaWikiTestCase {
 			$missingLocalFileRefs = $property->getValue( $module );
 
 			foreach ( $missingLocalFileRefs as $file ) {
-				$cases[] = array(
+				$cases[] = [
 					$file,
 					$moduleName,
 					$file,
-				);
+				];
 			}
 		}
 
